import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer as JSON_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_Uint16StrConversion as JSON_Deserializer_Uint16StrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_ByteStrConversion as JSON_Deserializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer as JSON_Deserializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_Spec as JSON_ConcreteSyntax_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_SpecProperties as JSON_ConcreteSyntax_SpecProperties
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax as JSON_ConcreteSyntax
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Serializer as JSON_ZeroCopy_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Core as JSON_ZeroCopy_Deserializer_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Strings as JSON_ZeroCopy_Deserializer_Strings

# Module: JSON_ZeroCopy_Deserializer_Numbers

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def Digits(cs):
        return ((cs).SkipWhile(JSON_Grammar.default__.Digit_q)).Split()

    @staticmethod
    def NonEmptyDigits(cs):
        d_0_sp_ = default__.Digits(cs)
        d_1_valueOrError0_ = Wrappers.default__.Need(not(((d_0_sp_).t).Empty_q), JSON_Utils_Cursors.CursorError_OtherError(JSON_Errors.DeserializationError_EmptyNumber()))
        if (d_1_valueOrError0_).IsFailure():
            return (d_1_valueOrError0_).PropagateFailure()
        elif True:
            return Wrappers.Result_Success(d_0_sp_)

    @staticmethod
    def NonZeroInt(cs):
        return default__.NonEmptyDigits(cs)

    @staticmethod
    def OptionalMinus(cs):
        def lambda0_(d_0_c_):
            return (d_0_c_) == (ord('-'))

        return ((cs).SkipIf(lambda0_)).Split()

    @staticmethod
    def OptionalSign(cs):
        def lambda0_(d_0_c_):
            return ((d_0_c_) == (ord('-'))) or ((d_0_c_) == (ord('+')))

        return ((cs).SkipIf(lambda0_)).Split()

    @staticmethod
    def TrimmedInt(cs):
        def lambda0_(d_1_c_):
            return (d_1_c_) == (ord('0'))

        d_0_sp_ = ((cs).SkipIf(lambda0_)).Split()
        if ((d_0_sp_).t).Empty_q:
            return default__.NonZeroInt((d_0_sp_).cs)
        elif True:
            return Wrappers.Result_Success(d_0_sp_)

    @staticmethod
    def Exp(cs):
        def lambda0_(d_0_c_):
            return ((d_0_c_) == (ord('e'))) or ((d_0_c_) == (ord('E')))

        let_tmp_rhs0_ = ((cs).SkipIf(lambda0_)).Split()
        d_1_e_ = let_tmp_rhs0_.t
        d_2_cs_ = let_tmp_rhs0_.cs
        if (d_1_e_).Empty_q:
            return Wrappers.Result_Success(JSON_Utils_Cursors.Split_SP(JSON_Grammar.Maybe_Empty(), d_2_cs_))
        elif True:
            let_tmp_rhs1_ = default__.OptionalSign(d_2_cs_)
            d_3_sign_ = let_tmp_rhs1_.t
            d_4_cs_ = let_tmp_rhs1_.cs
            d_5_valueOrError0_ = default__.NonEmptyDigits(d_4_cs_)
            if (d_5_valueOrError0_).IsFailure():
                return (d_5_valueOrError0_).PropagateFailure()
            elif True:
                let_tmp_rhs2_ = (d_5_valueOrError0_).Extract()
                d_6_num_ = let_tmp_rhs2_.t
                d_7_cs_ = let_tmp_rhs2_.cs
                return Wrappers.Result_Success(JSON_Utils_Cursors.Split_SP(JSON_Grammar.Maybe_NonEmpty(JSON_Grammar.jexp_JExp(d_1_e_, d_3_sign_, d_6_num_)), d_7_cs_))

    @staticmethod
    def Frac(cs):
        def lambda0_(d_0_c_):
            return (d_0_c_) == (ord('.'))

        let_tmp_rhs0_ = ((cs).SkipIf(lambda0_)).Split()
        d_1_period_ = let_tmp_rhs0_.t
        d_2_cs_ = let_tmp_rhs0_.cs
        if (d_1_period_).Empty_q:
            return Wrappers.Result_Success(JSON_Utils_Cursors.Split_SP(JSON_Grammar.Maybe_Empty(), d_2_cs_))
        elif True:
            d_3_valueOrError0_ = default__.NonEmptyDigits(d_2_cs_)
            if (d_3_valueOrError0_).IsFailure():
                return (d_3_valueOrError0_).PropagateFailure()
            elif True:
                let_tmp_rhs1_ = (d_3_valueOrError0_).Extract()
                d_4_num_ = let_tmp_rhs1_.t
                d_5_cs_ = let_tmp_rhs1_.cs
                return Wrappers.Result_Success(JSON_Utils_Cursors.Split_SP(JSON_Grammar.Maybe_NonEmpty(JSON_Grammar.jfrac_JFrac(d_1_period_, d_4_num_)), d_5_cs_))

    @staticmethod
    def NumberFromParts(minus, num, frac, exp):
        d_0_sp_ = JSON_Utils_Cursors.Split_SP(JSON_Grammar.jnumber_JNumber((minus).t, (num).t, (frac).t, (exp).t), (exp).cs)
        return d_0_sp_

    @staticmethod
    def Number(cs):
        d_0_minus_ = default__.OptionalMinus(cs)
        d_1_valueOrError0_ = default__.TrimmedInt((d_0_minus_).cs)
        if (d_1_valueOrError0_).IsFailure():
            return (d_1_valueOrError0_).PropagateFailure()
        elif True:
            d_2_num_ = (d_1_valueOrError0_).Extract()
            d_3_valueOrError1_ = default__.Frac((d_2_num_).cs)
            if (d_3_valueOrError1_).IsFailure():
                return (d_3_valueOrError1_).PropagateFailure()
            elif True:
                d_4_frac_ = (d_3_valueOrError1_).Extract()
                d_5_valueOrError2_ = default__.Exp((d_4_frac_).cs)
                if (d_5_valueOrError2_).IsFailure():
                    return (d_5_valueOrError2_).PropagateFailure()
                elif True:
                    d_6_exp_ = (d_5_valueOrError2_).Extract()
                    return Wrappers.Result_Success(default__.NumberFromParts(d_0_minus_, d_2_num_, d_4_frac_, d_6_exp_))




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer as JSON_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_Uint16StrConversion as JSON_Deserializer_Uint16StrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_ByteStrConversion as JSON_Deserializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer as JSON_Deserializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_Spec as JSON_ConcreteSyntax_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_SpecProperties as JSON_ConcreteSyntax_SpecProperties
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax as JSON_ConcreteSyntax
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Serializer as JSON_ZeroCopy_Serializer

# Module: JSON_ZeroCopy_Deserializer_Core

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def Get(cs, err):
        d_0_valueOrError0_ = (cs).Get(err)
        if (d_0_valueOrError0_).IsFailure():
            return (d_0_valueOrError0_).PropagateFailure()
        elif True:
            d_1_cs_ = (d_0_valueOrError0_).Extract()
            return Wrappers.Result_Success((d_1_cs_).Split())

    @staticmethod
    def WS(cs):
        sp: JSON_Utils_Cursors.Split = JSON_Utils_Cursors.Split.default(JSON_Grammar.jblanks.default)()
        d_0_point_k_: int
        d_0_point_k_ = (cs).point
        d_1_end_: int
        d_1_end_ = (cs).end
        while ((d_0_point_k_) < (d_1_end_)) and (JSON_Grammar.default__.Blank_q(((cs).s)[d_0_point_k_])):
            d_0_point_k_ = (d_0_point_k_) + (1)
        sp = (JSON_Utils_Cursors.Cursor___Cursor((cs).s, (cs).beg, d_0_point_k_, (cs).end)).Split()
        return sp
        return sp

    @staticmethod
    def Structural(cs, parser):
        let_tmp_rhs0_ = default__.WS(cs)
        d_0_before_ = let_tmp_rhs0_.t
        d_1_cs_ = let_tmp_rhs0_.cs
        d_2_valueOrError0_ = (parser).fn(d_1_cs_)
        if (d_2_valueOrError0_).IsFailure():
            return (d_2_valueOrError0_).PropagateFailure()
        elif True:
            let_tmp_rhs1_ = (d_2_valueOrError0_).Extract()
            d_3_val_ = let_tmp_rhs1_.t
            d_4_cs_ = let_tmp_rhs1_.cs
            let_tmp_rhs2_ = default__.WS(d_4_cs_)
            d_5_after_ = let_tmp_rhs2_.t
            d_6_cs_ = let_tmp_rhs2_.cs
            return Wrappers.Result_Success(JSON_Utils_Cursors.Split_SP(JSON_Grammar.Structural_Structural(d_0_before_, d_3_val_, d_5_after_), d_6_cs_))

    @staticmethod
    def TryStructural(cs):
        let_tmp_rhs0_ = default__.WS(cs)
        d_0_before_ = let_tmp_rhs0_.t
        d_1_cs_ = let_tmp_rhs0_.cs
        let_tmp_rhs1_ = ((d_1_cs_).SkipByte()).Split()
        d_2_val_ = let_tmp_rhs1_.t
        d_3_cs_ = let_tmp_rhs1_.cs
        let_tmp_rhs2_ = default__.WS(d_3_cs_)
        d_4_after_ = let_tmp_rhs2_.t
        d_5_cs_ = let_tmp_rhs2_.cs
        return JSON_Utils_Cursors.Split_SP(JSON_Grammar.Structural_Structural(d_0_before_, d_2_val_, d_4_after_), d_5_cs_)

    @_dafny.classproperty
    def SpecView(instance):
        def lambda0_(d_0_v_):
            return JSON_ConcreteSyntax_Spec.default__.View(d_0_v_)

        return lambda0_

class jopt:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return JSON_Utils_Views_Core.View__.OfBytes(_dafny.Seq([]))

class ValueParser:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return JSON_Utils_Parsers.SubParser.default()



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer as JSON_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_Uint16StrConversion as JSON_Deserializer_Uint16StrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_ByteStrConversion as JSON_Deserializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer as JSON_Deserializer

# Module: JSON_ConcreteSyntax_Spec

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def View(v):
        return (v).Bytes()

    @staticmethod
    def Structural(self, fT):
        return ((default__.View((self).before)) + (fT((self).t))) + (default__.View((self).after))

    @staticmethod
    def StructuralView(self):
        return default__.Structural(self, default__.View)

    @staticmethod
    def Maybe(self, fT):
        if (self).is_Empty:
            return _dafny.Seq([])
        elif True:
            return fT((self).t)

    @staticmethod
    def ConcatBytes(ts, fT):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (len(ts)) == (0):
                    return (d_0___accumulator_) + (_dafny.Seq([]))
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + (fT((ts)[0]))
                    in0_ = _dafny.Seq((ts)[1::])
                    in1_ = fT
                    ts = in0_
                    fT = in1_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def Bracketed(self, fDatum):
        return ((default__.StructuralView((self).l)) + (default__.ConcatBytes((self).data, fDatum))) + (default__.StructuralView((self).r))

    @staticmethod
    def KeyValue(self):
        return ((default__.String((self).k)) + (default__.StructuralView((self).colon))) + (default__.Value((self).v))

    @staticmethod
    def Frac(self):
        return (default__.View((self).period)) + (default__.View((self).num))

    @staticmethod
    def Exp(self):
        return ((default__.View((self).e)) + (default__.View((self).sign))) + (default__.View((self).num))

    @staticmethod
    def Number(self):
        return (((default__.View((self).minus)) + (default__.View((self).num))) + (default__.Maybe((self).frac, default__.Frac))) + (default__.Maybe((self).exp, default__.Exp))

    @staticmethod
    def String(self):
        return ((default__.View((self).lq)) + (default__.View((self).contents))) + (default__.View((self).rq))

    @staticmethod
    def CommaSuffix(c):
        return default__.Maybe(c, default__.StructuralView)

    @staticmethod
    def Member(self):
        return (default__.KeyValue((self).t)) + (default__.CommaSuffix((self).suffix))

    @staticmethod
    def Item(self):
        return (default__.Value((self).t)) + (default__.CommaSuffix((self).suffix))

    @staticmethod
    def Object(obj):
        def lambda0_(d_0_obj_):
            def lambda1_(d_1_d_):
                return default__.Member(d_1_d_)

            return lambda1_

        return default__.Bracketed(obj, lambda0_(obj))

    @staticmethod
    def Array(arr):
        def lambda0_(d_0_arr_):
            def lambda1_(d_1_d_):
                return default__.Item(d_1_d_)

            return lambda1_

        return default__.Bracketed(arr, lambda0_(arr))

    @staticmethod
    def Value(self):
        source0_ = self
        if True:
            if source0_.is_Null:
                d_0_n_ = source0_.n
                return default__.View(d_0_n_)
        if True:
            if source0_.is_Bool:
                d_1_b_ = source0_.b
                return default__.View(d_1_b_)
        if True:
            if source0_.is_String:
                d_2_str_ = source0_.str
                return default__.String(d_2_str_)
        if True:
            if source0_.is_Number:
                d_3_num_ = source0_.num
                return default__.Number(d_3_num_)
        if True:
            if source0_.is_Object:
                d_4_obj_ = source0_.obj
                return default__.Object(d_4_obj_)
        if True:
            d_5_arr_ = source0_.arr
            return default__.Array(d_5_arr_)

    @staticmethod
    def JSON(js):
        return default__.Structural(js, default__.Value)




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import TestUUID as TestUUID
import TestUTF8 as TestUTF8

# Module: TestTime

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def TestFormat():
        if not((Time.default__.FormatMilliDiff(123456, 123456)) == (_dafny.Seq("0.000"))):
            raise _dafny.HaltException("test/Time.dfy(11,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 123457)) == (_dafny.Seq("0.001"))):
            raise _dafny.HaltException("test/Time.dfy(12,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 123467)) == (_dafny.Seq("0.011"))):
            raise _dafny.HaltException("test/Time.dfy(13,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 123567)) == (_dafny.Seq("0.111"))):
            raise _dafny.HaltException("test/Time.dfy(14,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 124567)) == (_dafny.Seq("1.111"))):
            raise _dafny.HaltException("test/Time.dfy(15,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 134567)) == (_dafny.Seq("11.111"))):
            raise _dafny.HaltException("test/Time.dfy(16,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((Time.default__.FormatMilliDiff(123456, 234567)) == (_dafny.Seq("111.111"))):
            raise _dafny.HaltException("test/Time.dfy(17,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestNonDecreasing():
        d_0_t1_: int
        out0_: int
        out0_ = Time.default__.CurrentRelativeTime()
        d_0_t1_ = out0_
        d_1_t2_: int
        out1_: int
        out1_ = Time.default__.CurrentRelativeTime()
        d_1_t2_ = out1_
        if not((d_1_t2_) >= (d_0_t1_)):
            raise _dafny.HaltException("test/Time.dfy(23,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestNonDecreasingMilli():
        d_0_t1_: int
        out0_: int
        out0_ = Time.default__.CurrentRelativeTimeMilli()
        d_0_t1_ = out0_
        d_1_t2_: int
        out1_: int
        out1_ = Time.default__.CurrentRelativeTimeMilli()
        d_1_t2_ = out1_
        if not((d_1_t2_) >= (d_0_t1_)):
            raise _dafny.HaltException("test/Time.dfy(29,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestPositiveValues():
        d_0_t1_: int
        out0_: int
        out0_ = Time.default__.CurrentRelativeTime()
        d_0_t1_ = out0_
        d_1_t2_: int
        out1_: int
        out1_ = Time.default__.CurrentRelativeTime()
        d_1_t2_ = out1_
        if not(((d_1_t2_) - (d_0_t1_)) >= (0)):
            raise _dafny.HaltException("test/Time.dfy(35,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestGetCurrentTimeStamp():
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = Time.default__.GetCurrentTimeStamp()
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/Time.dfy(40,23): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_CurrentTime_: _dafny.Seq
        d_1_CurrentTime_ = (d_0_valueOrError0_).Extract()
        if not(default__.ISO8601_q(d_1_CurrentTime_)):
            raise _dafny.HaltException("test/Time.dfy(41,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def ISO8601_q(CreateTime):
        return ((((((((len(CreateTime)) == (27)) and (((CreateTime)[4]) == ('-'))) and (((CreateTime)[7]) == ('-'))) and (((CreateTime)[10]) == ('T'))) and (((CreateTime)[13]) == (':'))) and (((CreateTime)[16]) == (':'))) and (((CreateTime)[19]) == ('.'))) and (((CreateTime)[26]) == ('Z'))




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt

# Module: FloatCompare

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def StrToIntInner(s, acc):
        while True:
            with _dafny.label():
                if (len(s)) == (0):
                    return acc
                elif (('0') <= ((s)[0])) and (((s)[0]) <= ('9')):
                    in0_ = _dafny.Seq((s)[1::])
                    in1_ = (((acc) * (10)) + (ord((s)[0]))) - (ord('0'))
                    s = in0_
                    acc = in1_
                    raise _dafny.TailCall()
                elif True:
                    in2_ = _dafny.Seq((s)[1::])
                    in3_ = acc
                    s = in2_
                    acc = in3_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def SkipLeadingSpace(val):
        while True:
            with _dafny.label():
                if ((len(val)) > (0)) and (((val)[0]) <= (' ')):
                    in0_ = _dafny.Seq((val)[1::])
                    val = in0_
                    raise _dafny.TailCall()
                elif True:
                    return val
                break

    @staticmethod
    def StrToInt(s, acc):
        d_0_tmp_ = default__.SkipLeadingSpace(s)
        if (len(d_0_tmp_)) == (0):
            return 0
        elif ((d_0_tmp_)[0]) == ('-'):
            return (0) - (default__.StrToIntInner(s, 0))
        elif True:
            return default__.StrToIntInner(s, 0)

    @staticmethod
    def SplitE(x):
        d_0_parts_ = StandardLibrary.default__.SplitOnce_q(x, 'e')
        if (d_0_parts_).is_Some:
            return d_0_parts_
        elif True:
            return StandardLibrary.default__.SplitOnce_q(x, 'E')

    @staticmethod
    def SplitExp(x):
        d_0_parts_ = default__.SplitE(x)
        if (d_0_parts_).is_Some:
            return (((d_0_parts_).value)[0], default__.StrToInt(((d_0_parts_).value)[1], 0))
        elif True:
            return (x, 0)

    @staticmethod
    def SkipLeadingZeros(val):
        while True:
            with _dafny.label():
                if ((len(val)) > (0)) and (((val)[0]) == ('0')):
                    in0_ = _dafny.Seq((val)[1::])
                    val = in0_
                    raise _dafny.TailCall()
                elif True:
                    return val
                break

    @staticmethod
    def SkipTrailingZeros(val):
        while True:
            with _dafny.label():
                if ((len(val)) > (0)) and (((val)[(len(val)) - (1)]) == ('0')):
                    in0_ = _dafny.Seq((val)[:(len(val)) - (1):])
                    val = in0_
                    raise _dafny.TailCall()
                elif True:
                    return val
                break

    @staticmethod
    def SplitDot(x):
        d_0_parts_ = StandardLibrary.default__.SplitOnce_q(x, '.')
        if (d_0_parts_).is_Some:
            return (default__.SkipLeadingZeros(((d_0_parts_).value)[0]), default__.SkipTrailingZeros(((d_0_parts_).value)[1]))
        elif True:
            return (default__.SkipLeadingZeros(x), _dafny.Seq(""))

    @staticmethod
    def StrCmp(x, y):
        while True:
            with _dafny.label():
                if ((len(x)) == (0)) and ((len(y)) == (0)):
                    return 0
                elif (len(x)) == (0):
                    return -1
                elif (len(y)) == (0):
                    return 1
                elif ((x)[0]) < ((y)[0]):
                    return -1
                elif ((x)[0]) > ((y)[0]):
                    return 1
                elif True:
                    in0_ = _dafny.Seq((x)[1::])
                    in1_ = _dafny.Seq((y)[1::])
                    x = in0_
                    y = in1_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def AppendZeros(x, newLength):
        return (x) + (_dafny.Seq(['0' for d_0_i_ in range((newLength) - (len(x)))]))

    @staticmethod
    def CompareFloatInner(x, y):
        d_0_xParts_ = default__.SplitExp(x)
        d_1_yParts_ = default__.SplitExp(y)
        d_2_xNum_ = default__.SplitDot((d_0_xParts_)[0])
        d_3_yNum_ = default__.SplitDot((d_1_yParts_)[0])
        d_4_xDigits_ = default__.SkipLeadingZeros(((d_2_xNum_)[0]) + ((d_2_xNum_)[1]))
        d_5_yDigits_ = default__.SkipLeadingZeros(((d_3_yNum_)[0]) + ((d_3_yNum_)[1]))
        d_6_xExp_ = ((d_0_xParts_)[1]) - (len((d_2_xNum_)[1]))
        d_7_yExp_ = ((d_1_yParts_)[1]) - (len((d_3_yNum_)[1]))
        d_8_logX_ = (d_6_xExp_) + (len(d_4_xDigits_))
        d_9_logY_ = (d_7_yExp_) + (len(d_5_yDigits_))
        if (d_8_logX_) > (d_9_logY_):
            return 1
        elif (d_9_logY_) > (d_8_logX_):
            return -1
        elif (len(d_4_xDigits_)) < (len(d_5_yDigits_)):
            return default__.StrCmp(default__.AppendZeros(d_4_xDigits_, len(d_5_yDigits_)), d_5_yDigits_)
        elif (len(d_5_yDigits_)) < (len(d_4_xDigits_)):
            return default__.StrCmp(d_4_xDigits_, default__.AppendZeros(d_5_yDigits_, len(d_4_xDigits_)))
        elif True:
            return default__.StrCmp(d_4_xDigits_, d_5_yDigits_)

    @staticmethod
    def IsNegative(x):
        return ((len(x)) > (0)) and (((x)[0]) == ('-'))

    @staticmethod
    def SkipLeadingPlus(x):
        if ((0) < (len(x))) and (((x)[0]) == ('+')):
            return _dafny.Seq((x)[1::])
        elif True:
            return x

    @staticmethod
    def IsZero(x):
        while True:
            with _dafny.label():
                if (len(x)) == (0):
                    return True
                elif (((x)[0]) == ('0')) or (((x)[0]) == ('.')):
                    in0_ = _dafny.Seq((x)[1::])
                    x = in0_
                    raise _dafny.TailCall()
                elif (('1') <= ((x)[0])) and (((x)[0]) <= ('9')):
                    return False
                elif True:
                    return True
                break

    @staticmethod
    def RecognizeZero(x):
        if default__.IsNegative(x):
            if default__.IsZero(_dafny.Seq((x)[1::])):
                return _dafny.Seq("0")
            elif True:
                return x
        elif default__.IsZero(x):
            return _dafny.Seq("0")
        elif True:
            return x

    @staticmethod
    def CleanNumber(x):
        return default__.RecognizeZero(default__.SkipLeadingPlus(default__.SkipLeadingSpace(x)))

    @staticmethod
    def CompareFloat(x, y):
        d_0_x_ = default__.CleanNumber(x)
        d_1_y_ = default__.CleanNumber(y)
        if (default__.IsNegative(d_0_x_)) and (default__.IsNegative(d_1_y_)):
            return default__.CompareFloatInner(_dafny.Seq((d_1_y_)[1::]), _dafny.Seq((d_0_x_)[1::]))
        elif default__.IsNegative(d_0_x_):
            return -1
        elif default__.IsNegative(d_1_y_):
            return 1
        elif True:
            return default__.CompareFloatInner(d_0_x_, d_1_y_)

    @_dafny.classproperty
    def Less(instance):
        return -1
    @_dafny.classproperty
    def Equal(instance):
        return 0
    @_dafny.classproperty
    def Greater(instance):
        return 1

class CompareType:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_0_x_: int = source__
        return ((-1) <= (d_0_x_)) and ((d_0_x_) <= (1))



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq

# Module: JSON_Utils_Vectors

class default__:
    def  __init__(self):
        pass

    @_dafny.classproperty
    def OOM__FAILURE(instance):
        return Wrappers.Outcome_Fail(VectorError_OutOfMemory())

class VectorError:
    @_dafny.classproperty
    def AllSingletonConstructors(cls):
        return [VectorError_OutOfMemory()]
    @classmethod
    def default(cls, ):
        return lambda: VectorError_OutOfMemory()
    def __ne__(self, __o: object) -> bool:
        return not self.__eq__(__o)
    @property
    def is_OutOfMemory(self) -> bool:
        return isinstance(self, VectorError_OutOfMemory)

class VectorError_OutOfMemory(VectorError, NamedTuple('OutOfMemory', [])):
    def __dafnystr__(self) -> str:
        return f'Vectors.VectorError.OutOfMemory'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, VectorError_OutOfMemory)
    def __hash__(self) -> int:
        return super().__hash__()


class Vector:
    def  __init__(self):
        self.size: int = int(0)
        self.capacity: int = int(0)
        self.data: _dafny.Array = _dafny.Array(None, 0)
        self._a: TypeVar('A') = None
        pass

    def __dafnystr__(self) -> str:
        return "JSON.Utils.Vectors.Vector"
    def ctor__(self, a0, initial__capacity):
        (self)._a = a0
        (self).size = 0
        (self).capacity = initial__capacity
        def lambda0_(d_0_a0_):
            def lambda1_(d_1___v0_):
                return d_0_a0_

            return lambda1_

        init0_ = lambda0_(a0)
        nw0_ = _dafny.Array(None, initial__capacity)
        for i0_0_ in range(nw0_.length(0)):
            nw0_[i0_0_] = init0_(i0_0_)
        (self).data = nw0_

    def At(self, idx):
        return (self.data)[idx]

    def Top(self):
        return (self.data)[(self.size) - (1)]

    def Put(self, idx, a):
        arr0_ = self.data
        arr0_[(idx)] = a

    def CopyFrom(self, new__data, count):
        hi0_ = count
        for d_0_idx_ in range(0, hi0_):
            arr0_ = self.data
            arr0_[(d_0_idx_)] = (new__data)[d_0_idx_]

    def Realloc(self, new__capacity):
        d_0_old__data_: _dafny.Array
        d_1_old__capacity_: int
        rhs0_ = self.data
        rhs1_ = self.capacity
        d_0_old__data_ = rhs0_
        d_1_old__capacity_ = rhs1_
        def lambda0_(d_2___v1_):
            return (self).a

        init0_ = lambda0_
        nw0_ = _dafny.Array(None, new__capacity)
        for i0_0_ in range(nw0_.length(0)):
            nw0_[i0_0_] = init0_(i0_0_)
        rhs2_ = nw0_
        rhs3_ = new__capacity
        lhs0_ = self
        lhs1_ = self
        lhs0_.data = rhs2_
        lhs1_.capacity = rhs3_
        (self).CopyFrom(d_0_old__data_, d_1_old__capacity_)

    def DefaultNewCapacity(self, capacity):
        if (capacity) < ((self).MAX__CAPACITY__BEFORE__DOUBLING):
            return (2) * (capacity)
        elif True:
            return (self).MAX__CAPACITY

    def ReallocDefault(self):
        o: Wrappers.Outcome = Wrappers.Outcome.default()()
        if (self.capacity) == ((self).MAX__CAPACITY):
            o = Wrappers.Outcome_Fail(VectorError_OutOfMemory())
            return o
        (self).Realloc((self).DefaultNewCapacity(self.capacity))
        o = Wrappers.Outcome_Pass()
        return o
        return o

    def Ensure(self, reserved):
        o: Wrappers.Outcome = Wrappers.Outcome.default()()
        if (reserved) > (((self).MAX__CAPACITY) - (self.size)):
            o = Wrappers.Outcome_Fail(VectorError_OutOfMemory())
            return o
        if (reserved) <= ((self.capacity) - (self.size)):
            o = Wrappers.Outcome_Pass()
            return o
        d_0_new__capacity_: int
        d_0_new__capacity_ = self.capacity
        while (reserved) > ((d_0_new__capacity_) - (self.size)):
            d_0_new__capacity_ = (self).DefaultNewCapacity(d_0_new__capacity_)
        (self).Realloc(d_0_new__capacity_)
        o = Wrappers.Outcome_Pass()
        return o
        return o

    def PopFast(self):
        (self).size = (self.size) - (1)

    def PushFast(self, a):
        arr0_ = self.data
        index0_ = self.size
        arr0_[index0_] = a
        (self).size = (self.size) + (1)

    def Push(self, a):
        o: Wrappers.Outcome = Wrappers.Outcome.default()()
        if (self.size) == (self.capacity):
            d_0_d_: Wrappers.Outcome
            out0_: Wrappers.Outcome
            out0_ = (self).ReallocDefault()
            d_0_d_ = out0_
            if (d_0_d_).is_Fail:
                o = d_0_d_
                return o
        (self).PushFast(a)
        o = Wrappers.Outcome_Pass()
        return o
        return o

    @property
    def a(self):
        return self._a
    @property
    def MAX__CAPACITY__BEFORE__DOUBLING(self):
        return _dafny.euclidian_division(BoundedInts.default__.UINT32__MAX, 2)
    @property
    def MAX__CAPACITY(self):
        return BoundedInts.default__.UINT32__MAX



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar

# Module: JSON_Serializer_ByteStrConversion

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def Digits(n, base):
        if (n) == (0):
            return _dafny.Seq([])
        elif True:
            d_0_digits_k_ = default__.Digits(_dafny.euclidian_division(n, base), base)
            d_1_digits_ = (d_0_digits_k_) + (_dafny.Seq([_dafny.euclidian_modulus(n, base)]))
            return d_1_digits_

    @staticmethod
    def OfDigits(digits, chars):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (digits) == (_dafny.Seq([])):
                    return (d_0___accumulator_) + (_dafny.Seq([]))
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + (_dafny.Seq([(chars)[(digits)[0]]]))
                    in0_ = _dafny.Seq((digits)[1::])
                    in1_ = chars
                    digits = in0_
                    chars = in1_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def OfNat__any(n, chars):
        d_0_base_ = len(chars)
        if (n) == (0):
            return _dafny.Seq([(chars)[0]])
        elif True:
            return default__.OfDigits(default__.Digits(n, d_0_base_), chars)

    @staticmethod
    def NumberStr(str, minus, is__digit):
        def lambda0_(forall_var_0_):
            d_0_c_: int = forall_var_0_
            return not ((d_0_c_) in (_dafny.Seq((str)[1::]))) or (is__digit(d_0_c_))

        return not ((str) != (_dafny.Seq([]))) or (((((str)[0]) == (minus)) or (is__digit((str)[0]))) and (_dafny.quantifier((_dafny.Seq((str)[1::])).UniqueElements, True, lambda0_)))

    @staticmethod
    def OfInt__any(n, chars, minus):
        if (n) >= (0):
            return default__.OfNat__any(n, chars)
        elif True:
            return (_dafny.Seq([minus])) + (default__.OfNat__any((0) - (n), chars))

    @staticmethod
    def ToNat__any(str, base, digits):
        if (str) == (_dafny.Seq([])):
            return 0
        elif True:
            return ((default__.ToNat__any(_dafny.Seq((str)[:(len(str)) - (1):]), base, digits)) * (base)) + ((digits)[(str)[(len(str)) - (1)]])

    @staticmethod
    def ToInt__any(str, minus, base, digits):
        if (_dafny.Seq([minus])) <= (str):
            return (0) - (default__.ToNat__any(_dafny.Seq((str)[1::]), base, digits))
        elif True:
            return default__.ToNat__any(str, base, digits)




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer as JSON_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_Uint16StrConversion as JSON_Deserializer_Uint16StrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_ByteStrConversion as JSON_Deserializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer as JSON_Deserializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_Spec as JSON_ConcreteSyntax_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_SpecProperties as JSON_ConcreteSyntax_SpecProperties
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax as JSON_ConcreteSyntax

# Module: JSON_ZeroCopy_Serializer

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def Serialize(js):
        rbs: Wrappers.Result = Wrappers.Result.default(_dafny.defaults.pointer)()
        d_0_writer_: JSON_Utils_Views_Writers.Writer__
        d_0_writer_ = default__.Text(js)
        d_1_valueOrError0_: Wrappers.Outcome = Wrappers.Outcome.default()()
        d_1_valueOrError0_ = Wrappers.default__.Need((d_0_writer_).Unsaturated_q, JSON_Errors.SerializationError_OutOfMemory())
        if (d_1_valueOrError0_).IsFailure():
            rbs = (d_1_valueOrError0_).PropagateFailure()
            return rbs
        d_2_bs_: _dafny.Array
        out0_: _dafny.Array
        out0_ = (d_0_writer_).ToArray()
        d_2_bs_ = out0_
        rbs = Wrappers.Result_Success(d_2_bs_)
        return rbs
        return rbs

    @staticmethod
    def SerializeTo(js, dest):
        len: Wrappers.Result = Wrappers.Result.default(BoundedInts.uint32.default)()
        d_0_writer_: JSON_Utils_Views_Writers.Writer__
        d_0_writer_ = default__.Text(js)
        d_1_valueOrError0_: Wrappers.Outcome = Wrappers.Outcome.default()()
        d_1_valueOrError0_ = Wrappers.default__.Need((d_0_writer_).Unsaturated_q, JSON_Errors.SerializationError_OutOfMemory())
        if (d_1_valueOrError0_).IsFailure():
            len = (d_1_valueOrError0_).PropagateFailure()
            return len
        d_2_valueOrError1_: Wrappers.Outcome = Wrappers.Outcome.default()()
        d_2_valueOrError1_ = Wrappers.default__.Need(((d_0_writer_).length) <= ((dest).length(0)), JSON_Errors.SerializationError_OutOfMemory())
        if (d_2_valueOrError1_).IsFailure():
            len = (d_2_valueOrError1_).PropagateFailure()
            return len
        (d_0_writer_).CopyTo(dest)
        len = Wrappers.Result_Success((d_0_writer_).length)
        return len
        return len

    @staticmethod
    def Text(js):
        return default__.JSON(js, JSON_Utils_Views_Writers.Writer__.Empty)

    @staticmethod
    def JSON(js, writer):
        def lambda0_(d_0_js_):
            def lambda1_(d_1_wr_):
                return default__.Value((d_0_js_).t, d_1_wr_)

            return lambda1_

        return (((writer).Append((js).before)).Then(lambda0_(js))).Append((js).after)

    @staticmethod
    def Value(v, writer):
        source0_ = v
        if True:
            if source0_.is_Null:
                d_0_n_ = source0_.n
                return (writer).Append(d_0_n_)
        if True:
            if source0_.is_Bool:
                d_1_b_ = source0_.b
                d_2_wr_ = (writer).Append(d_1_b_)
                return d_2_wr_
        if True:
            if source0_.is_String:
                d_3_str_ = source0_.str
                d_4_wr_ = default__.String(d_3_str_, writer)
                return d_4_wr_
        if True:
            if source0_.is_Number:
                d_5_num_ = source0_.num
                d_6_wr_ = default__.Number(d_5_num_, writer)
                return d_6_wr_
        if True:
            if source0_.is_Object:
                d_7_obj_ = source0_.obj
                d_8_wr_ = default__.Object(d_7_obj_, writer)
                return d_8_wr_
        if True:
            d_9_arr_ = source0_.arr
            d_10_wr_ = default__.Array(d_9_arr_, writer)
            return d_10_wr_

    @staticmethod
    def String(str, writer):
        return (((writer).Append((str).lq)).Append((str).contents)).Append((str).rq)

    @staticmethod
    def Number(num, writer):
        d_0_wr_ = ((writer).Append((num).minus)).Append((num).num)
        d_1_wr_ = (((d_0_wr_).Append((((num).frac).t).period)).Append((((num).frac).t).num) if ((num).frac).is_NonEmpty else d_0_wr_)
        d_2_wr_ = ((((d_1_wr_).Append((((num).exp).t).e)).Append((((num).exp).t).sign)).Append((((num).exp).t).num) if ((num).exp).is_NonEmpty else d_1_wr_)
        return d_2_wr_

    @staticmethod
    def StructuralView(st, writer):
        return (((writer).Append((st).before)).Append((st).t)).Append((st).after)

    @staticmethod
    def Object(obj, writer):
        d_0_wr_ = default__.StructuralView((obj).l, writer)
        d_1_wr_ = default__.Members(obj, d_0_wr_)
        d_2_wr_ = default__.StructuralView((obj).r, d_1_wr_)
        return d_2_wr_

    @staticmethod
    def Array(arr, writer):
        d_0_wr_ = default__.StructuralView((arr).l, writer)
        d_1_wr_ = default__.Items(arr, d_0_wr_)
        d_2_wr_ = default__.StructuralView((arr).r, d_1_wr_)
        return d_2_wr_

    @staticmethod
    def Members(obj, writer):
        wr: JSON_Utils_Views_Writers.Writer__ = JSON_Utils_Views_Writers.Writer.default()
        out0_: JSON_Utils_Views_Writers.Writer__
        out0_ = default__.MembersImpl(obj, writer)
        wr = out0_
        return wr

    @staticmethod
    def Items(arr, writer):
        wr: JSON_Utils_Views_Writers.Writer__ = JSON_Utils_Views_Writers.Writer.default()
        out0_: JSON_Utils_Views_Writers.Writer__
        out0_ = default__.ItemsImpl(arr, writer)
        wr = out0_
        return wr

    @staticmethod
    def MembersImpl(obj, writer):
        wr: JSON_Utils_Views_Writers.Writer__ = JSON_Utils_Views_Writers.Writer.default()
        wr = writer
        d_0_members_: _dafny.Seq
        d_0_members_ = (obj).data
        hi0_ = len(d_0_members_)
        for d_1_i_ in range(0, hi0_):
            wr = default__.Member((d_0_members_)[d_1_i_], wr)
        return wr

    @staticmethod
    def ItemsImpl(arr, writer):
        wr: JSON_Utils_Views_Writers.Writer__ = JSON_Utils_Views_Writers.Writer.default()
        wr = writer
        d_0_items_: _dafny.Seq
        d_0_items_ = (arr).data
        hi0_ = len(d_0_items_)
        for d_1_i_ in range(0, hi0_):
            wr = default__.Item((d_0_items_)[d_1_i_], wr)
        return wr

    @staticmethod
    def Member(m, writer):
        d_0_wr_ = default__.String(((m).t).k, writer)
        d_1_wr_ = default__.StructuralView(((m).t).colon, d_0_wr_)
        d_2_wr_ = default__.Value(((m).t).v, d_1_wr_)
        d_3_wr_ = (d_2_wr_ if ((m).suffix).is_Empty else default__.StructuralView(((m).suffix).t, d_2_wr_))
        return d_3_wr_

    @staticmethod
    def Item(m, writer):
        d_0_wr_ = default__.Value((m).t, writer)
        d_1_wr_ = (d_0_wr_ if ((m).suffix).is_Empty else default__.StructuralView(((m).suffix).t, d_0_wr_))
        return d_1_wr_




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq

# Module: BoundedInts

class default__:
    def  __init__(self):
        pass

    @_dafny.classproperty
    def TWO__TO__THE__8(instance):
        return 256
    @_dafny.classproperty
    def UINT8__MAX(instance):
        return 255
    @_dafny.classproperty
    def TWO__TO__THE__16(instance):
        return 65536
    @_dafny.classproperty
    def UINT16__MAX(instance):
        return 65535
    @_dafny.classproperty
    def TWO__TO__THE__32(instance):
        return 4294967296
    @_dafny.classproperty
    def UINT32__MAX(instance):
        return 4294967295
    @_dafny.classproperty
    def TWO__TO__THE__64(instance):
        return 18446744073709551616
    @_dafny.classproperty
    def UINT64__MAX(instance):
        return 18446744073709551615
    @_dafny.classproperty
    def INT8__MIN(instance):
        return -128
    @_dafny.classproperty
    def INT8__MAX(instance):
        return 127
    @_dafny.classproperty
    def INT16__MIN(instance):
        return -32768
    @_dafny.classproperty
    def INT16__MAX(instance):
        return 32767
    @_dafny.classproperty
    def INT32__MIN(instance):
        return -2147483648
    @_dafny.classproperty
    def INT32__MAX(instance):
        return 2147483647
    @_dafny.classproperty
    def INT64__MIN(instance):
        return -9223372036854775808
    @_dafny.classproperty
    def INT64__MAX(instance):
        return 9223372036854775807
    @_dafny.classproperty
    def NAT8__MAX(instance):
        return 127
    @_dafny.classproperty
    def NAT16__MAX(instance):
        return 32767
    @_dafny.classproperty
    def NAT32__MAX(instance):
        return 2147483647
    @_dafny.classproperty
    def NAT64__MAX(instance):
        return 9223372036854775807
    @_dafny.classproperty
    def TWO__TO__THE__0(instance):
        return 1
    @_dafny.classproperty
    def TWO__TO__THE__1(instance):
        return 2
    @_dafny.classproperty
    def TWO__TO__THE__2(instance):
        return 4
    @_dafny.classproperty
    def TWO__TO__THE__4(instance):
        return 16
    @_dafny.classproperty
    def TWO__TO__THE__5(instance):
        return 32
    @_dafny.classproperty
    def TWO__TO__THE__24(instance):
        return 16777216
    @_dafny.classproperty
    def TWO__TO__THE__40(instance):
        return 1099511627776
    @_dafny.classproperty
    def TWO__TO__THE__48(instance):
        return 281474976710656
    @_dafny.classproperty
    def TWO__TO__THE__56(instance):
        return 72057594037927936
    @_dafny.classproperty
    def TWO__TO__THE__128(instance):
        return 340282366920938463463374607431768211456
    @_dafny.classproperty
    def TWO__TO__THE__256(instance):
        return 115792089237316195423570985008687907853269984665640564039457584007913129639936
    @_dafny.classproperty
    def TWO__TO__THE__512(instance):
        return 13407807929942597099574024998205846127479365820592393377723561443721764030073546976801874298166903427690031858186486050853753882811946569946433649006084096

class uint8:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class uint16:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class uint32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class uint64:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class int8:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class int16:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class int32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class int64:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        return True

class nat8:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_0_x_: int = source__
        return ((0) <= (d_0_x_)) and ((d_0_x_) < (128))

class nat16:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_1_x_: int = source__
        return ((0) <= (d_1_x_)) and ((d_1_x_) < (32768))

class nat32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_2_x_: int = source__
        return ((0) <= (d_2_x_)) and ((d_2_x_) < (2147483648))

class nat64:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_3_x_: int = source__
        return ((0) <= (d_3_x_)) and ((d_3_x_) < (9223372036854775808))

class opt__byte:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return int(0)
    def _Is(source__):
        d_4_c_: int = source__
        return ((-1) <= (d_4_c_)) and ((d_4_c_) < (default__.TWO__TO__THE__8))



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views

# Module: JSON_Utils_Lexers_Core


class LexerResult:
    @classmethod
    def default(cls, ):
        return lambda: LexerResult_Accept()
    def __ne__(self, __o: object) -> bool:
        return not self.__eq__(__o)
    @property
    def is_Accept(self) -> bool:
        return isinstance(self, LexerResult_Accept)
    @property
    def is_Reject(self) -> bool:
        return isinstance(self, LexerResult_Reject)
    @property
    def is_Partial(self) -> bool:
        return isinstance(self, LexerResult_Partial)

class LexerResult_Accept(LexerResult, NamedTuple('Accept', [])):
    def __dafnystr__(self) -> str:
        return f'Core.LexerResult.Accept'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, LexerResult_Accept)
    def __hash__(self) -> int:
        return super().__hash__()

class LexerResult_Reject(LexerResult, NamedTuple('Reject', [('err', Any)])):
    def __dafnystr__(self) -> str:
        return f'Core.LexerResult.Reject({_dafny.string_of(self.err)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, LexerResult_Reject) and self.err == __o.err
    def __hash__(self) -> int:
        return super().__hash__()

class LexerResult_Partial(LexerResult, NamedTuple('Partial', [('st', Any)])):
    def __dafnystr__(self) -> str:
        return f'Core.LexerResult.Partial({_dafny.string_of(self.st)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, LexerResult_Partial) and self.st == __o.st
    def __hash__(self) -> int:
        return super().__hash__()




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions

# Module: Utf8EncodingForm

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def IsMinimalWellFormedCodeUnitSubsequence(s):
        if (len(s)) == (1):
            d_0_b_ = default__.IsWellFormedSingleCodeUnitSequence(s)
            return d_0_b_
        elif (len(s)) == (2):
            d_1_b_ = default__.IsWellFormedDoubleCodeUnitSequence(s)
            return d_1_b_
        elif (len(s)) == (3):
            d_2_b_ = default__.IsWellFormedTripleCodeUnitSequence(s)
            return d_2_b_
        elif (len(s)) == (4):
            d_3_b_ = default__.IsWellFormedQuadrupleCodeUnitSequence(s)
            return d_3_b_
        elif True:
            return False

    @staticmethod
    def IsWellFormedSingleCodeUnitSequence(s):
        d_0_firstByte_ = (s)[0]
        return (True) and (((0) <= (d_0_firstByte_)) and ((d_0_firstByte_) <= (127)))

    @staticmethod
    def IsWellFormedDoubleCodeUnitSequence(s):
        d_0_firstByte_ = (s)[0]
        d_1_secondByte_ = (s)[1]
        return (((194) <= (d_0_firstByte_)) and ((d_0_firstByte_) <= (223))) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191)))

    @staticmethod
    def IsWellFormedTripleCodeUnitSequence(s):
        d_0_firstByte_ = (s)[0]
        d_1_secondByte_ = (s)[1]
        d_2_thirdByte_ = (s)[2]
        return ((((((d_0_firstByte_) == (224)) and (((160) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191)))) or ((((225) <= (d_0_firstByte_)) and ((d_0_firstByte_) <= (236))) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191))))) or (((d_0_firstByte_) == (237)) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (159))))) or ((((238) <= (d_0_firstByte_)) and ((d_0_firstByte_) <= (239))) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191))))) and (((128) <= (d_2_thirdByte_)) and ((d_2_thirdByte_) <= (191)))

    @staticmethod
    def IsWellFormedQuadrupleCodeUnitSequence(s):
        d_0_firstByte_ = (s)[0]
        d_1_secondByte_ = (s)[1]
        d_2_thirdByte_ = (s)[2]
        d_3_fourthByte_ = (s)[3]
        return ((((((d_0_firstByte_) == (240)) and (((144) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191)))) or ((((241) <= (d_0_firstByte_)) and ((d_0_firstByte_) <= (243))) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (191))))) or (((d_0_firstByte_) == (244)) and (((128) <= (d_1_secondByte_)) and ((d_1_secondByte_) <= (143))))) and (((128) <= (d_2_thirdByte_)) and ((d_2_thirdByte_) <= (191)))) and (((128) <= (d_3_fourthByte_)) and ((d_3_fourthByte_) <= (191)))

    @staticmethod
    def SplitPrefixMinimalWellFormedCodeUnitSubsequence(s):
        if ((len(s)) >= (1)) and (default__.IsWellFormedSingleCodeUnitSequence(_dafny.Seq((s)[:1:]))):
            return Wrappers.Option_Some(_dafny.Seq((s)[:1:]))
        elif ((len(s)) >= (2)) and (default__.IsWellFormedDoubleCodeUnitSequence(_dafny.Seq((s)[:2:]))):
            return Wrappers.Option_Some(_dafny.Seq((s)[:2:]))
        elif ((len(s)) >= (3)) and (default__.IsWellFormedTripleCodeUnitSequence(_dafny.Seq((s)[:3:]))):
            return Wrappers.Option_Some(_dafny.Seq((s)[:3:]))
        elif ((len(s)) >= (4)) and (default__.IsWellFormedQuadrupleCodeUnitSequence(_dafny.Seq((s)[:4:]))):
            return Wrappers.Option_Some(_dafny.Seq((s)[:4:]))
        elif True:
            return Wrappers.Option_None()

    @staticmethod
    def EncodeScalarValue(v):
        if (v) <= (127):
            return default__.EncodeScalarValueSingleByte(v)
        elif (v) <= (2047):
            return default__.EncodeScalarValueDoubleByte(v)
        elif (v) <= (65535):
            return default__.EncodeScalarValueTripleByte(v)
        elif True:
            return default__.EncodeScalarValueQuadrupleByte(v)

    @staticmethod
    def EncodeScalarValueSingleByte(v):
        d_0_x_ = (v) & (127)
        d_1_firstByte_ = d_0_x_
        return _dafny.Seq([d_1_firstByte_])

    @staticmethod
    def EncodeScalarValueDoubleByte(v):
        d_0_x_ = (v) & (63)
        d_1_y_ = ((v) & (1984)) >> (6)
        d_2_firstByte_ = (192) | (d_1_y_)
        d_3_secondByte_ = (128) | (d_0_x_)
        return _dafny.Seq([d_2_firstByte_, d_3_secondByte_])

    @staticmethod
    def EncodeScalarValueTripleByte(v):
        d_0_x_ = (v) & (63)
        d_1_y_ = ((v) & (4032)) >> (6)
        d_2_z_ = ((v) & (61440)) >> (12)
        d_3_firstByte_ = (224) | (d_2_z_)
        d_4_secondByte_ = (128) | (d_1_y_)
        d_5_thirdByte_ = (128) | (d_0_x_)
        return _dafny.Seq([d_3_firstByte_, d_4_secondByte_, d_5_thirdByte_])

    @staticmethod
    def EncodeScalarValueQuadrupleByte(v):
        d_0_x_ = (v) & (63)
        d_1_y_ = ((v) & (4032)) >> (6)
        d_2_z_ = ((v) & (61440)) >> (12)
        d_3_u2_ = ((v) & (196608)) >> (16)
        d_4_u1_ = ((v) & (1835008)) >> (18)
        d_5_firstByte_ = (240) | (d_4_u1_)
        d_6_secondByte_ = ((128) | (((d_3_u2_) << (4)) & ((1 << 8) - 1))) | (d_2_z_)
        d_7_thirdByte_ = (128) | (d_1_y_)
        d_8_fourthByte_ = (128) | (d_0_x_)
        return _dafny.Seq([d_5_firstByte_, d_6_secondByte_, d_7_thirdByte_, d_8_fourthByte_])

    @staticmethod
    def DecodeMinimalWellFormedCodeUnitSubsequence(m):
        if (len(m)) == (1):
            return default__.DecodeMinimalWellFormedCodeUnitSubsequenceSingleByte(m)
        elif (len(m)) == (2):
            return default__.DecodeMinimalWellFormedCodeUnitSubsequenceDoubleByte(m)
        elif (len(m)) == (3):
            return default__.DecodeMinimalWellFormedCodeUnitSubsequenceTripleByte(m)
        elif True:
            return default__.DecodeMinimalWellFormedCodeUnitSubsequenceQuadrupleByte(m)

    @staticmethod
    def DecodeMinimalWellFormedCodeUnitSubsequenceSingleByte(m):
        d_0_firstByte_ = (m)[0]
        d_1_x_ = d_0_firstByte_
        return d_1_x_

    @staticmethod
    def DecodeMinimalWellFormedCodeUnitSubsequenceDoubleByte(m):
        d_0_firstByte_ = (m)[0]
        d_1_secondByte_ = (m)[1]
        d_2_y_ = (d_0_firstByte_) & (31)
        d_3_x_ = (d_1_secondByte_) & (63)
        return (((d_2_y_) << (6)) & ((1 << 24) - 1)) | (d_3_x_)

    @staticmethod
    def DecodeMinimalWellFormedCodeUnitSubsequenceTripleByte(m):
        d_0_firstByte_ = (m)[0]
        d_1_secondByte_ = (m)[1]
        d_2_thirdByte_ = (m)[2]
        d_3_z_ = (d_0_firstByte_) & (15)
        d_4_y_ = (d_1_secondByte_) & (63)
        d_5_x_ = (d_2_thirdByte_) & (63)
        return ((((d_3_z_) << (12)) & ((1 << 24) - 1)) | (((d_4_y_) << (6)) & ((1 << 24) - 1))) | (d_5_x_)

    @staticmethod
    def DecodeMinimalWellFormedCodeUnitSubsequenceQuadrupleByte(m):
        d_0_firstByte_ = (m)[0]
        d_1_secondByte_ = (m)[1]
        d_2_thirdByte_ = (m)[2]
        d_3_fourthByte_ = (m)[3]
        d_4_u1_ = (d_0_firstByte_) & (7)
        d_5_u2_ = ((d_1_secondByte_) & (48)) >> (4)
        d_6_z_ = (d_1_secondByte_) & (15)
        d_7_y_ = (d_2_thirdByte_) & (63)
        d_8_x_ = (d_3_fourthByte_) & (63)
        return ((((((d_4_u1_) << (18)) & ((1 << 24) - 1)) | (((d_5_u2_) << (16)) & ((1 << 24) - 1))) | (((d_6_z_) << (12)) & ((1 << 24) - 1))) | (((d_7_y_) << (6)) & ((1 << 24) - 1))) | (d_8_x_)

    @staticmethod
    def PartitionCodeUnitSequenceChecked(s):
        maybeParts: Wrappers.Option = Wrappers.Option.default()()
        if (s) == (_dafny.Seq([])):
            maybeParts = Wrappers.Option_Some(_dafny.Seq([]))
            return maybeParts
        d_0_result_: _dafny.Seq
        d_0_result_ = _dafny.Seq([])
        d_1_rest_: _dafny.Seq
        d_1_rest_ = s
        while (len(d_1_rest_)) > (0):
            d_2_valueOrError0_: Wrappers.Option = Wrappers.Option.default()()
            d_2_valueOrError0_ = default__.SplitPrefixMinimalWellFormedCodeUnitSubsequence(d_1_rest_)
            if (d_2_valueOrError0_).IsFailure():
                maybeParts = (d_2_valueOrError0_).PropagateFailure()
                return maybeParts
            d_3_prefix_: _dafny.Seq
            d_3_prefix_ = (d_2_valueOrError0_).Extract()
            d_0_result_ = (d_0_result_) + (_dafny.Seq([d_3_prefix_]))
            d_1_rest_ = _dafny.Seq((d_1_rest_)[len(d_3_prefix_)::])
        maybeParts = Wrappers.Option_Some(d_0_result_)
        return maybeParts
        return maybeParts

    @staticmethod
    def PartitionCodeUnitSequence(s):
        return (default__.PartitionCodeUnitSequenceChecked(s)).Extract()

    @staticmethod
    def IsWellFormedCodeUnitSequence(s):
        return (default__.PartitionCodeUnitSequenceChecked(s)).is_Some

    @staticmethod
    def EncodeScalarSequence(vs):
        s: _dafny.Seq = WellFormedCodeUnitSeq.default()
        s = _dafny.Seq([])
        lo0_ = 0
        for d_0_i_ in range(len(vs)-1, lo0_-1, -1):
            d_1_next_: _dafny.Seq
            d_1_next_ = default__.EncodeScalarValue((vs)[d_0_i_])
            s = (d_1_next_) + (s)
        return s

    @staticmethod
    def DecodeCodeUnitSequence(s):
        d_0_parts_ = default__.PartitionCodeUnitSequence(s)
        d_1_vs_ = Seq.default__.Map(default__.DecodeMinimalWellFormedCodeUnitSubsequence, d_0_parts_)
        return d_1_vs_

    @staticmethod
    def DecodeCodeUnitSequenceChecked(s):
        maybeVs: Wrappers.Option = Wrappers.Option.default()()
        d_0_maybeParts_: Wrappers.Option
        d_0_maybeParts_ = default__.PartitionCodeUnitSequenceChecked(s)
        if (d_0_maybeParts_).is_None:
            maybeVs = Wrappers.Option_None()
            return maybeVs
        d_1_parts_: _dafny.Seq
        d_1_parts_ = (d_0_maybeParts_).value
        d_2_vs_: _dafny.Seq
        d_2_vs_ = Seq.default__.Map(default__.DecodeMinimalWellFormedCodeUnitSubsequence, d_1_parts_)
        maybeVs = Wrappers.Option_Some(d_2_vs_)
        return maybeVs
        return maybeVs


class WellFormedCodeUnitSeq:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq([])
    def _Is(source__):
        d_3_s_: _dafny.Seq = source__
        return default__.IsWellFormedCodeUnitSequence(d_3_s_)

class MinimalWellFormedCodeUnitSeq:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq({})
    def _Is(source__):
        d_4_s_: _dafny.Seq = source__
        return default__.IsMinimalWellFormedCodeUnitSubsequence(d_4_s_)
